@@ -3,7 +3,7 @@ from django.views.generic.edit import CreateView
 from django.forms.models import modelform_factory, inlineformset_factory, \
     BaseInlineFormSet, ModelForm, _get_foreign_key, modelformset_factory
 from django.template.loader import render_to_string
-from django.contrib.contenttypes.generic import generic_inlineformset_factory
+from django.contrib.contenttypes.forms import generic_inlineformset_factory
 from django.shortcuts import redirect
 from django.utils.translation import ugettext_lazy as _
 
@@ -15,14 +15,17 @@ class Inline(object):
     form_class = None
     formset_class = None
 
-    def __init__(self, model, form=None, template="forms/tabular_inline.html",
-        **kwargs):
+    def __init__(
+        self, model, form=None, formset_class=None,
+        template="forms/inline.html", **kwargs
+    ):
 
         self.model = model
         self.opts = self.model._meta
         self.template = template
 
         self.form_class = form
+        self.formset_class = formset_class
         self.form_kwargs = self.form_kwargs.copy()
         self.formset_kwargs = self.formset_kwargs.copy()
         self.formset_kwargs.update(kwargs)
@@ -74,7 +77,14 @@ class Inline(object):
             **self.formset_kwargs)
 
     def __unicode__(self):
-        return render_to_string(self.template, {"inline": self})
+        return render_to_string(
+            self.template,
+            {
+                "formset": self.formset,
+                "title": self.opts.verbose_name_plural.title,
+                "inline": self,
+            }
+        )
 
 
 class GenericInline(Inline):
@@ -121,6 +131,7 @@ class ModelFormWithInlinesView(CreateView):
     otherwise an object will be created
     """
     inlines = []
+    fields = '__all__'
 
     def get_inlines(self):
         return self.inlines
