@@ -1,14 +1,16 @@
 from __future__ import print_function
-from django.core.urlresolvers import reverse
-from django.db.models.fields.related import ReverseSingleRelatedObjectDescriptor
+
+import six
+from django.db import models
+from django.db.models.base import ModelBase
+from django.db.models.fields import FieldDoesNotExist
+from django.urls import reverse
 from django.utils.encoding import python_2_unicode_compatible
+from django.utils.html import escape
 from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext as _
 
-from viewsets.mixins.base import SessionDataMixin
-import six
-from django.db.models.fields import FieldDoesNotExist
-from django.utils.html import escapejs, escape
+from .base import SessionDataMixin
 
 
 @python_2_unicode_compatible
@@ -123,13 +125,13 @@ class TableField(object):
         return None
 
     def value(self, instance):
-        return unicode(instance)
+        return six.text_type(instance)
 
 
 class UnicodeTableField(TableField):
 
     def valid(self):
-        return self.field == "__unicode__" or self.field == "__str__"
+        return self.field == "__str__" or self.field == "__str__"
 
     def header(self):
         try:
@@ -138,7 +140,7 @@ class UnicodeTableField(TableField):
             return ""
 
     def sort(self):
-        return ":default:"
+        return ""
 
 
 class CallableTableField(TableField):
@@ -185,18 +187,28 @@ class ModelTableField(CallableTableField):
         if isinstance(self.field, six.string_types):
             field = self.view.model
             for subfield in self.field.split("__"):
-                item = getattr(field, subfield, None)
+                item = None
 
-                if isinstance(item, ReverseSingleRelatedObjectDescriptor):
-                    item = item.get_query_set().model
+                if isinstance(field, ModelBase):
+                    item = field._meta.get_field(subfield)
 
-                if not item:
-                    try:
-                        # if its a field, then get it's verbose name
-                        item = field._meta.get_field(subfield).verbose_name
-                        self.is_model_field = True
-                    except FieldDoesNotExist:
-                        pass
+                    if getattr(item, 'related_model', None):
+                        item = item.related_model
+
+                    else:
+                        try:
+                            if hasattr(item, 'get_queryset'):
+                                item = item.get_queryset().model
+                        except (Exception, FieldDoesNotExist):
+                            pass
+
+                        if not item:
+                            try:
+                                # if its a field, then get it's verbose name
+                                item = field._meta.get_field(subfield).verbose_name
+                                self.is_model_field = True
+                            except (AttributeError, FieldDoesNotExist):
+                                pass
 
                 field = item
 
@@ -234,7 +246,7 @@ class ModelTableField(CallableTableField):
 
 class TableMixin(SortMixin):
     """ causes views with a list to have enough context to make a table """
-    list_display = ["__unicode__"]
+    list_display = ["__str__"]
     list_display_links = []
     list_editable = None  # NOT Implemented
     list_detail_link = ""
@@ -309,9 +321,9 @@ class TableMixin(SortMixin):
                 retval = field.value(obj)
             except Exception as ex:
                 print (type(ex), ex)
-                retval = u"<i style='color:darkred;' " + \
+                retval = mark_safe(u"<i style='color:darkred;' " + \
                     u"class='glyphicon glyphicon-exclamation-sign' " + \
-                    u"title='{}: {}'></i>".format(type(ex).__name__, escape(str(ex)))
+                    u"title='{}: {}'></i>".format(type(ex).__name__, escape(str(ex))))
 
             if retval is None:
                 retval = "_"
